//////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Some Code ported from:
//  WIRELESS MOUSETRAP MONITORING SYSTEM by Alberto Ricci Bitti (c) 2003
//  www.riccibitti.com
//
//////////////////////////////////////////////////////////////////////////////////////////////////////
//
// RXM_TX684_Hardw_R0.h
//
// Hardware register/pin configuration
//
//////////////////////////////////////////////////////////////////////////////////////////////////////
//
//      Hardware Configuration:
//
//                          PIC12F684
//
//                       +-------_-------+
//              VCC  - 1 | Vdd       Vss | 14 - GND
//                       |               |
//        TX DAT OUT - 2 | RA5       RA0 | 13 - PGD J1-4 (Free)
//                       |               |
//        TX PWR DWN - 3 | RA4       RA1 | 12 - PGC, VREF INPUT
//                       |               |
//   MR- SWITCH1 INP - 4 | RA3       RA2 | 11 - SWITCH2 INPUT
//                       |               |
//      SETUP SWITCH - 5 | RC5   RC0/AN4 | 10 - ANALOG INPUT 0-15VDC
//                       |               |
//         LED DRIVE - 6 | RC4   RC1/AN5 | 9  - KTY81 INPUT
//                       |               |
//     +VCC SWITCHED - 7 | RC3   RC2/AN6 | 8  - VBAT ANALOG INPUT
//                       +---------------+
//

//*****************************************************************************
//* Required header files
//*****************************************************************************

#include "16F684.h"    // Device header file
#device ADC=10

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

/******************************************************************************/
/*Setup for delay, serial port, device fuses, and interrupt priority          */
/******************************************************************************/

// Factory Cal value at address 03FF => 0x345C for PIC12F683 used

#use delay(clock=4000000, RESTART_WDT)

#fuses INTRC_IO, NOPROTECT, WDT, NOMCLR, NOBROWNOUT, NOPUT

#use rs232(baud=9600,xmit=PIN_A0,rcv=PIN_A3,RESTART_WDT)


/******************************************************************************/
/* special function register addresses                                        */
/******************************************************************************/

#byte PORTA  = 0x05      // GPIO data register
#byte PORTC  = 0x06      // GPIO data register

#byte ADCON0 = 0x1F     // ADC Control register
#byte ADCON1 = 0x9F		// ADC Clock Control Register
#bit GO_DONE = 0x1F.1   // 1 = starts ADC conversion, 1 -> 0 ADC conversion done
#bit ADC_CTL = 0x1F.0	// 1 = ADC ON, 0 = ADC OFF

#byte ADRESH = 0x1E     // ADC MSB data register
#byte ADRESL = 0x9E     // ADC LSB data register
#byte ANSEL  = 0x91     // ADC Channel select register


/******************************************************************************/
/* IO pin definitions                                                         */
/******************************************************************************/

// *********************** Port A *************************************

#define FREE1           PIN_A0  // PGD - FREE on J1 pin-4 (RS232 OUT)
#define AN1_VREF        PIN_A1  // PGC - VREF INPUT from LM385 - 2.2VDC
#define INPUT2          PIN_A2  // SWITCH2 INPUT, Active low
#define INPUT1          PIN_A3  // VPP -  SWITCH1 INPUT, Active low
#define PDN_OUT		    PIN_A4  // TX PD CTL OUTPUT
#define TX_OUTPUT       PIN_A5  // TX DATA OUTPUT

// *********************** Port C *************************************

#define EXT_AN_INP		PIN_C0 // EXT ANALOG INPUT 0-15VDC
#define KTY81_INP   	PIN_C1 // KTY81 ANALOG INPUT
#define VBAT_AN_INP		PIN_C2 // VBAT ANALOG INPUT
#define SW_SYS_PWR		PIN_C3 // SWITCHED VCC
#define LED_OUT			PIN_C4 // LED DRIVE ACTIVE LOW
#define SETUP_SW		PIN_C5 // SETUP SWITCH INPUT

#use fast_io(a)
#define PORT_A_MASK 0b00001110      //Port A direction register mask, 1 = input
#define PORT_C_MASK 0b00100111      //Port C direction register mask, 1 = input

#define KTY81_CHANNEL 1

/******************************************************************************/
/* TIMER0 re-load                                                             */
/******************************************************************************/

///////////////////////////////////////////////////////////////////////////////
// Note: the actual set_timer0 value must be corrected to allow to correct
// the time the compiler has used to save the CPU registers. This is for
// the 683 chip exactly 16 instructions (see list file location 0x004 - 0x040)
// the true load value is: 98 to 106
// See timer0 interrupt function
///////////////////////////////////////////////////////////////////////////////

#define T0_LOAD 98    // for a 167us interrupt overflow rate 89 + 16

#define PAUSE_TIME_mS 12    //minimum pause duration between TX repetitions
#define HEARTBEAT_SECONDS (60 * 60) //60 sec * 60 min = transmit every hour
#define TX_ON 1
#define TX_OFF 0
#define TX_INTERVALL 1
#define PACKET_SIZE 8
#define START_BIT 0x80
#define NUM_ID 16

/******************************************************************************/
/* Macro Definitions                                                          */
/******************************************************************************/

// This macro waits for Timer0 interrupt causing TX_FLAG to be set
#define TX_WAIT 	while(TX_FLAG == FALSE); TX_Flag = FALSE;

#define TX_START 	output_bit(TX_OUTPUT, TX_ON);
#define TX_STOP 	output_bit(TX_OUTPUT, TX_OFF);

#define PDN_HIGH	output_bit(PDN_OUT, 1);
#define PDN_LOW		output_bit(PDN_OUT, 0);

#define SYSPWR_ON	output_bit(SW_SYS_PWR, 1);
#define SYSPWR_OFF	output_bit(SW_SYS_PWR, 0);

#define ADC_WAIT 	while(GO_DONE == 1) delay_us(1);

#define ADC_ON 		ADC_CTL = 1;
#define ADC_SD   	ADC_CTL = 0;

#define START_ADC_CONVERSION	GO_DONE = TRUE;

#define LED_ON      output_bit(LED_OUT, 0);
#define LED_OFF     output_bit(LED_OUT, 1);

/******************************************************************************/
/*Global variables                                                            */
/******************************************************************************/

long analogs[]={0,0,0,0};

//*****************************************************************************
// Flags
//*****************************************************************************

short TX_FLAG = 0;

/******************************************************************************/
/*Function prototypes                                                         */
/******************************************************************************/


void system_334us_timer(void);

void new_xmit( char repetitions, char *_rp);
void hardware_initialize(void);
void read_ADCvolts(char *_rp);

static unsigned int get_interval( unsigned char ID);

// eof
